@@ -1,58 +1,59 @@
 /* ====================================================================
- * Copyright (c) 1995-1999 The Apache Group.  All rights reserved.
+ * The Apache Software License, Version 1.1
+ *
+ * Copyright (c) 2000 The Apache Software Foundation.  All rights
+ * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
+ *    notice, this list of conditions and the following disclaimer.
  *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in
  *    the documentation and/or other materials provided with the
  *    distribution.
  *
- * 3. All advertising materials mentioning features or use of this
- *    software must display the following acknowledgment:
- *    "This product includes software developed by the Apache Group
- *    for use in the Apache HTTP server project (http://www.apache.org/)."
+ * 3. The end-user documentation included with the redistribution,
+ *    if any, must include the following acknowledgment:
+ *       "This product includes software developed by the
+ *        Apache Software Foundation (http://www.apache.org/)."
+ *    Alternately, this acknowledgment may appear in the software itself,
+ *    if and wherever such third-party acknowledgments normally appear.
  *
- * 4. The names "Apache Server" and "Apache Group" must not be used to
- *    endorse or promote products derived from this software without
- *    prior written permission. For written permission, please contact
- *    apache@apache.org.
+ * 4. The names "Apache" and "Apache Software Foundation" must
+ *    not be used to endorse or promote products derived from this
+ *    software without prior written permission. For written
+ *    permission, please contact apache@apache.org.
  *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Group.
+ * 5. Products derived from this software may not be called "Apache",
+ *    nor may "Apache" appear in their name, without prior written
+ *    permission of the Apache Software Foundation.
  *
- * 6. Redistributions of any form whatsoever must retain the following
- *    acknowledgment:
- *    "This product includes software developed by the Apache Group
- *    for use in the Apache HTTP server project (http://www.apache.org/)."
- *
- * THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
- * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
  * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
- * OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
+ * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
+ * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  * ====================================================================
  *
  * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Group and was originally based
- * on public domain software written at the National Center for
- * Supercomputing Applications, University of Illinois, Urbana-Champaign.
- * For more information on the Apache Group and the Apache HTTP server
- * project, please see <http://www.apache.org/>.
+ * individuals on behalf of the Apache Software Foundation.  For more
+ * information on the Apache Software Foundation, please see
+ * <http://www.apache.org/>.
  *
+ * Portions of this software are based upon public domain software
+ * originally written at the National Center for Supercomputing Applications,
+ * University of Illinois, Urbana-Champaign.
  */
 
 #include "apache_cookie.h"
@@ -62,23 +63,22 @@ char *ApacheCookie_expires(ApacheCookie *c, char *time_str)
     char *expires;
 
     expires = ApacheUtil_expires(c->r->pool, time_str, EXPIRES_COOKIE);
-    if (expires) {
+    if (expires)
 	c->expires = expires;
-    }
 
     return c->expires;
 }
 
 #define cookie_get_set(thing,val) \
-retval = thing; \
-if(val) thing = ap_pstrdup(c->r->pool, val)
+    retval = thing; \
+    if(val) thing = ap_pstrdup(c->r->pool, val)
 
 char *ApacheCookie_attr(ApacheCookie *c, char *key, char *val)
 {
     char *retval = NULL;
     int ix = key[0] == '-' ? 1 : 0;
 
-    switch (key[ix]) { 
+    switch (key[ix]) {
     case 'n':
 	cookie_get_set(c->name, val);
 	break;
@@ -96,14 +96,14 @@ char *ApacheCookie_attr(ApacheCookie *c, char *key, char *val)
 	break;
     case 's':
 	if(val) {
-	    c->secure = 
+	    c->secure =
 		!strcaseEQ(val, "off") &&
 		!strcaseEQ(val, "0");
 	}
 	retval = c->secure ? "on" : "";
 	break;
     default:
-	ap_log_rerror(APC_ERROR, 
+	ap_log_rerror(APC_ERROR,
 		      "[libapreq] unknown cookie pair: `%s' => `%s'", key, val);
     };
 
@@ -114,7 +114,7 @@ ApacheCookie *ApacheCookie_new(request_rec *r, ...)
 {
     va_list args;
     ApacheRequest req;
-    ApacheCookie *c = 
+    ApacheCookie *c =
 	ap_pcalloc(r->pool, sizeof(ApacheCookie));
 
     req.r = r;
@@ -130,9 +130,9 @@ ApacheCookie *ApacheCookie_new(request_rec *r, ...)
     for(;;) {
 	char *key, *val;
 	key = va_arg(args, char *);
-	if (key == NULL) {
+	if (key == NULL)
 	    break;
-	}
+
 	val = va_arg(args, char *);
 	(void)ApacheCookie_attr(c, key, val);
     }
@@ -144,38 +144,39 @@ ApacheCookie *ApacheCookie_new(request_rec *r, ...)
 ApacheCookieJar *ApacheCookie_parse(request_rec *r, const char *data)
 {
     const char *pair;
-    ApacheCookieJar *retval = 
+    ApacheCookieJar *retval =
 	ap_make_array(r->pool, 1, sizeof(ApacheCookie *));
 
-    if (!data) {
-	if (!(data = ap_table_get(r->headers_in, "Cookie"))) {
+    if (!data)
+	if (!(data = ap_table_get(r->headers_in, "Cookie")))
 	    return retval;
-	}
-    }
 
     while (*data && (pair = ap_getword(r->pool, &data, ';'))) {
 	const char *key, *val;
 	ApacheCookie *c;
 
-	while (ap_isspace(*data)) {
+	while (ap_isspace(*data))
 	    ++data;
-	}
+
 	key = ap_getword(r->pool, &pair, '=');
 	ap_unescape_url((char *)key);
 	c = ApacheCookie_new(r, "-name", key, NULL);
-	if (c->values) {
+
+	if (c->values)
 	    c->values->nelts = 0;
-	}
-	else {
+	else
 	    c->values = ap_make_array(r->pool, 4, sizeof(char *));
-	}
 
-	if (!*pair) {
+	if (!*pair)
 	    ApacheCookieAdd(c, "");
-	}
 
-	while (*pair && (val = ap_getword(r->pool, &pair, '&'))) {
+
+	while (*pair && (val = ap_getword_nulls(r->pool, &pair, '&'))) {
 	    ap_unescape_url((char *)val);
+#ifdef DEBUG
+	    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r, 
+                          "[apache_cookie] added (%s)", val);
+#endif
 	    ApacheCookieAdd(c, val);
 	}
 	ApacheCookieJarAdd(retval, c);
@@ -188,16 +189,17 @@ ApacheCookieJar *ApacheCookie_parse(request_rec *r, const char *data)
     *(char **)ap_push_array(arr) = (char *)val
 
 #define cookie_push_named(arr, name, val) \
-    if(val) { \
+    if(val && strlen(val) > 0) { \
         cookie_push_arr(arr, ap_pstrcat(p, name, "=", val, NULL)); \
     }
 
-static char * escape_url(pool *p, char *val) 
+static char * escape_url(pool *p, char *val)
 {
   char *result = ap_os_escape_path(p, val?val:"", 1);
   char *end = result + strlen(result);
   char *seek;
 
+  /* touchup result to ensure that special chars are escaped */
   for ( seek = end-1; seek >= result; --seek) {
     char *ptr, *replacement;
 
@@ -215,10 +217,8 @@ static char * escape_url(pool *p, char *val)
 	continue; /* next for() */
     }
 
-
-    for (ptr = end; ptr > seek; --ptr) {
-      ptr[2] = ptr[0];
-    }
+    for (ptr = end; ptr > seek; --ptr)
+        ptr[2] = ptr[0];
 
     strncpy(seek, replacement, 3);
     end += 2;
@@ -234,9 +234,8 @@ char *ApacheCookie_as_string(ApacheCookie *c)
     char *cookie, *retval;
     int i;
 
-    if (!c->name) {
+    if (!c->name)
 	return "";
-    }
 
     values = ap_make_array(p, 6, sizeof(char *));
     cookie_push_named(values, "domain",  c->domain);
@@ -248,15 +247,15 @@ char *ApacheCookie_as_string(ApacheCookie *c)
 
     cookie = ap_pstrcat(p, escape_url(p, c->name), "=", NULL);
     for (i=0; i<c->values->nelts; i++) {
-	cookie = ap_pstrcat(p, cookie, 
-			    escape_url(p, ((char**)c->values->elts)[i]), 
-			    (i < (c->values->nelts-1) ? "&" : NULL),
+	cookie = ap_pstrcat(p, cookie,
+			    escape_url(p, ((char**)c->values->elts)[i]),
+			    (i < (c->values->nelts -1) ? "&" : NULL),
 			    NULL);
     }
 
     retval = cookie;
     for (i=0; i<values->nelts; i++) {
-	retval = ap_pstrcat(p, retval, "; ", 
+	retval = ap_pstrcat(p, retval, "; ",
 			    ((char**)values->elts)[i], NULL);
     }
 
